#ifndef ELEVATORGUI_HPP
#define ELEVATORGUI_HPP

#include <gtkmm.h>
#include <iostream>
#include <iomanip>
#include <vector>
#include <sstream>

#include "elevatorcontroller.hpp"

#include "elevatordoor.hpp"
#include "callbutton.hpp"
#include "positionlabel.hpp"
#include "requestbutton.hpp"


typedef std::vector<CallButton*> CallButtonVector;
typedef std::vector<PositionLabel*> PositionLabelVector;
typedef std::vector<RequestButton*> RequestButtonVector;
typedef std::vector<ElevatorDoor*> ElevatorDoorVector;


class ElevatorGUI : public Gtk::Window
{
	public:
		ElevatorGUI (int floors, int elevators);

		/* Functions to be called from Elevator to change GUI status */
		void gui_update_position_label (int elevator, float new_position, Direction direction);
		void gui_unpress_call_button (int floor, Direction direction);
		void gui_unpress_request_button (int elevator, int floor);
		void gui_open_door (int elevator, int floor);
		void gui_close_door (int elevator, int floor);

	private:
		/* Callbacks from button presses */
		void on_call_button_toggled (CallButton *button);
		void on_request_button_toggled (RequestButton *button);
		void on_playpause_button_clicked ();
		void on_quit_button_clicked ();

		/* Timer Function */
		bool on_timer_tick ();
		sigc::connection timer_;
		static const int timer_tick_ms_ = 500;

		int number_of_floors_;
		int number_of_elevators_;

		enum { RUNNING, PAUSED } simulation_status_;
		
		ElevatorController ec_;

		// holds custom CallButton which knows it's direction and floor#
		CallButtonVector call_buttons_;

		// holds custom position label which knows it's elevator#
		PositionLabelVector position_labels_;

		// holds custom RequestButton which knows it's elevator# and floor#
		RequestButtonVector request_buttons_;

		// holds custom ElevatorDoor which knows it's elevator# and floor#
		ElevatorDoorVector elevator_doors_;

		// Holds the Play / Pause button and Quit button
		Gtk::Button button_playpause_, button_quit_;

		// Holds the Table which holds everything
		Gtk::Table table_;
};

#endif /* ELEVATORGUI_HPP */

/* vim: set ts=4 sts=4 sw=4 noet tw=112: */
